@@ -366,7 +366,7 @@ def test_case_hidden(test_case_command):
 
 
 def supply_input(inputs):
-    """Specifies the input for a test case.
+    """Specifies the input for a test case (adds newline at end).
 
     Arguments:
         *inputs: inputs to be used for test case
@@ -374,8 +374,21 @@ def supply_input(inputs):
     Returns:
         None
     """
-    with open("fileinput", "a") as outfile:
-        outfile.write(inputs)
+    with open("fileinput", "ab") as outfile:
+        outfile.write((inputs + "\n").encode("utf-8"))
+
+
+def supply_input_bare(inputs):
+    """Specifies the input for a test case without adding a newline.
+
+    Arguments:
+        *inputs: inputs to be used for test case
+
+    Returns:
+        None
+    """
+    with open("fileinput", "ab") as outfile:
+        outfile.write(inputs.encode("utf-8"))
 
 
 def supply_input_file(input_file):
@@ -387,15 +400,15 @@ def supply_input_file(input_file):
     Returns:
         None
     """
-    with open(input_file, "r") as infile:
-        input_lines = infile.readlines()
+    with open(input_file, "rb") as infile:
+        input_data = infile.read()
 
-    with open("fileinput", "a") as outfile:
-        outfile.writelines(input_lines)
+    with open("fileinput", "ab") as outfile:
+        outfile.write(input_data)
 
 
 def check_output(outputs):
-    """Specifies the expected output for a test case.
+    """Specifies the expected output for a test case (adds newline at end).
 
     Arguments:
         *outputs: outputs to be used for test case
@@ -408,6 +421,20 @@ def check_output(outputs):
         outfile.write(outputs + "\n")
 
 
+def check_output_bare(outputs):
+    """Specifies the expected output for a test case without adding a newline.
+
+    Arguments:
+        *outputs: outputs to be used for test case
+
+    Returns:
+        None
+    """
+
+    with open("expectedoutput", "a") as outfile:
+        outfile.write(outputs)
+
+
 def check_output_file(output_file):
     """Specifies the expected output for a test case read from a file.
 
@@ -425,7 +452,7 @@ def check_output_file(output_file):
 
 
 def check_error(error_output):
-    """Specifies the expected error output for a test case.
+    """Specifies the expected error output for a test case (adds newline at end).
 
     Arguments:
         error_output: expected error output for a test case
@@ -437,6 +464,19 @@ def check_error(error_output):
         outfile.write(error_output + "\n")
 
 
+def check_error_bare(error_output):
+    """Specifies the expected error output for a test case without adding a newline.
+
+    Arguments:
+        error_output: expected error output for a test case
+
+    Returns:
+        None
+    """
+    with open("expectederror", "a") as outfile:
+        outfile.write(error_output)
+
+
 def hint(hints):
     """Hint
 
@@ -536,10 +576,13 @@ tag_func_map = {
     "T": test_case,
     "HT": test_case_hidden,
     "I": supply_input,
+    "IB": supply_input_bare,
     "IF": supply_input_file,
     "O": check_output,
+    "OB": check_output_bare,
     "OF": check_output_file,
     "E": check_error,
+    "EB": check_error_bare,
     "HINT": hint,
     "TO": timeout,
     "X": exit_code,
@@ -584,6 +627,8 @@ def parse_tags(tags: list[str]):
     tag_only_pattern = r"([A-Z_]+)\s*$"
 
     valid_tags = set(tag_func_map.keys())
+    # Tags to silently ignore (used by other tools but not by run-evaluation)
+    ignored_tags = {"CTO", "Z", "RUN"}
 
     # Track if we're inside a CRT_HW block (content to ignore)
     in_crt_hw_block = False
@@ -609,6 +654,9 @@ def parse_tags(tags: list[str]):
         # Check for tag without arguments
         if tag_only_match and not tag_match:
             tag = tag_only_match.group(1)
+            # Skip ignored tags
+            if tag in ignored_tags:
+                continue
             if tag in valid_tags:
                 print(f"Error on line {line_num}: Tag '{tag}' requires arguments")
                 print(f"  {line_num}: {tag_line.rstrip()}")
@@ -626,6 +674,9 @@ def parse_tags(tags: list[str]):
             potential_tag = re.match(r"([A-Z]+)", tag_line)
             if potential_tag:
                 tag = potential_tag.group(1)
+                # Skip ignored tags
+                if tag in ignored_tags:
+                    continue
                 if tag not in valid_tags and len(tag) <= 4:
                     print(f"Error on line {line_num}: Unknown tag '{tag}'")
                     print(f"  {line_num}: {tag_line.rstrip()}")
@@ -636,6 +687,10 @@ def parse_tags(tags: list[str]):
         tag = tag_match.group(1)
         args = tag_match.group(2)
 
+        # Skip ignored tags
+        if tag in ignored_tags:
+            continue
+
         # Check for unknown tag
         if tag not in valid_tags:
             print(f"Error on line {line_num}: Unknown tag '{tag}'")
@@ -715,7 +770,7 @@ def check_test():
     print(f"Test case {test_case_count} of {test_case_total}")
     passed = True
 
-    with open("fileinput", "r") as fileinput, open(
+    with open("fileinput", "rb") as fileinput, open(
         "youroutput", "w"
     ) as youroutput, open("yourerror", "w") as yourerror:
         test_exec = subprocess.Popen(
